# -*- coding: ISO-8859-1 -*-

# Copyright (C) 2002, 2003 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# uncomment next line for pychecker
# import gettext; gettext.install("PyTone", "")

import curses, curses.panel, fcntl, os.path, struct, termios
import events, hub
import filelistwin
import playlistwin
import playerwin
import iteminfowin
import helpwin
import statusbar
import timer
import config

from helper import debug

class mainscreen:
    def __init__(self, screen, songdbids, playerids):
        self.screen = screen
        self.h, self.w = screen.getmaxyx()

        leftpanelw = int(self.w/2.2)
        rightpanelw = self.w-leftpanelw

        self.channel = hub.hub.newchannel()
        self.timer = timer.timer()

        self.keybindings = config.keybindings.general

        self.statusbar = statusbar.statusbar(screen,
                                             self.h-1, self.w,
                                             self.channel)

        self.filelistwin = filelistwin.filelistwin(screen,
                                                   self.h-1, leftpanelw,
                                                   0, 0,
                                                   self.channel,
                                                   songdbids)

        self.playlistwin = playlistwin.playlistwin(screen,
                                                   self.h-6-3-1, rightpanelw,
                                                   9, leftpanelw,
                                                   self.channel,
                                                   "main")

        if config.inputwindow.type=="popup":
            from inputwin import popupinputwin as inputwin
        else:
            from inputwin import statusbarinputwin as inputwin

        self.inputwin = inputwin(screen,
                                 self.h, self.w,
                                 self.channel)

        self.playerwin = playerwin.playerwin(screen,
                                             3, rightpanelw, 0, leftpanelw,
                                             self.channel,
                                             playerids[0])

        self.iteminfowin = iteminfowin.iteminfowin(screen,
                                                   6, rightpanelw, 3, leftpanelw,
                                                   self.channel,
                                                   playerids[1])

        self.helpwin = helpwin.helpwin(self.screen,
                                       self.h, self.w,
                                       self.channel,
                                       self.timer)

        if config.mixer.device:
            if config.mixerwindow.type=="popup":
                from mixerwin import popupmixerwin as mixerwin
            else:
                from mixerwin import statusbarmixerwin as mixerwin
            self.mixerwin = mixerwin(screen,
                                     self.h, self.w,
                                     self.channel,
                                     self.timer)

        else:
            # disbable keybindings to obtain correct help window contents
            del config.keybindings.general.volumeup
            del config.keybindings.general.volumedown

        self.channel.subscribe(events.keypressed, self.keypressed)
        self.channel.subscribe(events.activateplaylist, self.activateplaylist)
        self.channel.subscribe(events.activatefilelist, self.activatefilelist)

        hub.hub.notify(events.activatefilelist())

        # try to load dump from prior crash, if existent
        if config.general.dumpfile:
            try:
                if os.path.isfile(config.general.dumpfile):
                    self.load()
                    os.unlink(config.general.dumpfile)
            except:
                pass

    def run(self):
        """ main loop of control thread """
        skipcount = 0
        while 1:
            try:
                key = self.screen.getch()

                if key==27:
                    # handle escape sequence (e.g. alt+key)
                    key = self.screen.getch()+1024

                if key==curses.KEY_MOUSE:
                    mouse = curses.getmouse()
                    x, y = mouse[1:3]
                    state = mouse[4]
                    hub.hub.notify(events.mouseevent(y, x, state))
                elif key==curses.KEY_RESIZE:
                    self.resizeterminal()
                elif key in self.keybindings["exit"]:
                    curses.halfdelay(5)
                    key = self.screen.getch()
                    curses.halfdelay(1)
                    if key in self.keybindings["exit"]:
                        break
                elif key!=-1:
                    hub.hub.notify(events.keypressed(key), 50)

                self.timer.process()
                self.channel.process()

                # update screen
                if key == -1 or skipcount>=config.general.throttleoutput:
                    curses.panel.update_panels()
                    curses.doupdate()
                    skipcount = 0
                else:
                    skipcount += 1

            except KeyboardInterrupt:
                pass

    def dump(self):
        """ dump state of PyTone """
        dumpfile = open(config.general.dumpfile, "w")
        self.playlistwin.dump(dumpfile)

    def load(self):
        """ dump state of PyTone """
        dumpfile = open(config.general.dumpfile, "r")
        self.playlistwin.load(dumpfile)

    def quit(self):
        """ cleanup """
        hub.hub.notify(events.quit(), 100)

    def getmaxyx(self):
        # taken from http://groups.google.com/groups?selm=uk7xtwasm.fsf%40python.net
        try:
            return int(os.environ["LINES"]), int(os.environ["COLUMNS"])
        except KeyError:
            height, width = struct.unpack(
                "hhhh", fcntl.ioctl(0, termios.TIOCGWINSZ ,"\000"*8))[0:2]
            if not height: return 25, 80
            return height, width

    def refresh(self):
        # refresh screen (i don't know any better way...)
        self.screen.clear()
        self.filelistwin.update()
        self.playlistwin.update()
        self.playerwin.update()
        self.iteminfowin.update()
        self.statusbar.update()

    def resizeterminal(self):
        self.h, self.w = self.getmaxyx()

        leftpanelw = int(self.w/2.2)
        rightpanelw = self.w-leftpanelw

        self.filelistwin.resize(self.h-1, leftpanelw, 0, 0)
        self.playerwin.resize(3, rightpanelw, 0, leftpanelw)
        self.iteminfowin.resize(6, rightpanelw, 3, leftpanelw)
        self.playlistwin.resize(self.h-6-3-1, rightpanelw, 9, leftpanelw)
        self.statusbar.resize(self.h-1, self.w)
        self.helpwin.resize(self.h, self.w)
        self.inputwin.resize(self.h, self.w)

        if self.__dict__.has_key("mixerwin"):
            self.mixerwin.resize(self.h, self.w)

        self.filelistwin.update()
        self.playlistwin.update()
        self.playerwin.update()
        self.iteminfowin.update()
        self.statusbar.update()

    # event handler

    def keypressed(self, event):
        key = event.key
        if key in self.keybindings["refresh"]:
            self.refresh()
        elif key in self.keybindings["playlistdeleteplayedsongs"]:
            hub.hub.notify(events.playlistdeleteplayedsongs())
        elif key in self.keybindings["playlistclear"]:
            hub.hub.notify(events.playlistclear())
            hub.hub.notify(events.activatefilelist())
        elif key in self.keybindings["playlistreplay"]:
            hub.hub.notify(events.playlistreplay())
        elif key in self.keybindings["playlistload"]:
            hub.hub.notify(events.playlistload())
        elif key in self.keybindings["playlistsave"]:
            hub.hub.notify(events.playlistsave())
        elif key in self.keybindings["playlisttoggleautoplaymode"]:
            hub.hub.notify(events.playlisttoggleautoplaymode())
        elif key in self.keybindings["showhelp"]:
            if self.filelistwin.hasfocus():
                context = "filelistwindow"
            elif self.playlistwin.hasfocus():
                context = "playlistwindow"
            else:
                context = None

            self.helpwin.showhelp(context)
        else:
            debug("unknown key: %d\n" % key)

    def activateplaylist(self, event):
        self.playlistwin.top()

    def activatefilelist(self, event):
        self.filelistwin.top()
